@@ -1,57 +1,69 @@
+# This file is part of the django-environ.
+#
+# Copyright (c) 2021, Serghei Iakovlev <egrep@protonmail.ch>
+# Copyright (c) 2013-2021, Daniele Faraglia <daniele.faraglia@gmail.com>
+#
+# For the full copyright and license information, please view
+# the LICENSE.txt file that was distributed with this source code.
+
 """
 Django-environ allows you to utilize 12factor inspired environment
 variables to configure your Django application.
 """
-import json
+
+import ast
 import logging
 import os
 import re
 import sys
+import urllib.parse as urlparselib
 import warnings
+from urllib.parse import (
+    parse_qs,
+    ParseResult,
+    unquote_plus,
+    urlparse,
+    urlunparse,
+)
+
+from .compat import DJANGO_POSTGRES, ImproperlyConfigured, json, REDIS_DRIVER
 
 try:
-    from django.core.exceptions import ImproperlyConfigured
+    from os import PathLike
+    Openable = (str, PathLike)
 except ImportError:
-    class ImproperlyConfigured(Exception):
-        pass
+    Openable = (str,)
 
-from six.moves import urllib
-from six import string_types
 
 logger = logging.getLogger(__name__)
 
 
-VERSION = '0.4.4'
-__author__ = 'joke2k'
-__version__ = tuple(VERSION.split('.'))
+def _cast(value):
+    # Safely evaluate an expression node or a string containing a Python
+    # literal or container display.
+    # https://docs.python.org/3/library/ast.html#ast.literal_eval
+    try:
+        return ast.literal_eval(value)
+    except ValueError:
+        return value
 
 
-# return int if possible
 def _cast_int(v):
+    """Return int if possible."""
     return int(v) if hasattr(v, 'isdigit') and v.isdigit() else v
 
+
 def _cast_urlstr(v):
-    return urllib.parse.unquote_plus(v) if isinstance(v, str) else v
-
-# back compatibility with redis_cache package
-DJANGO_REDIS_DRIVER = 'django_redis.cache.RedisCache'
-DJANGO_REDIS_CACHE_DRIVER = 'redis_cache.RedisCache'
-
-REDIS_DRIVER = DJANGO_REDIS_DRIVER
-try:
-    import redis_cache
-    REDIS_DRIVER = DJANGO_REDIS_CACHE_DRIVER
-except:
-    pass
+    return unquote_plus(v) if isinstance(v, str) else v
 
 
-class NoValue(object):
+class NoValue:
 
     def __repr__(self):
-        return '<{0}>'.format(self.__class__.__name__)
+        return '<{}>'.format(self.__class__.__name__)
 
 
-class Env(object):
+class Env:
 
     """Provide scheme-based lookups of environment variables so that each
     caller doesn't have to pass in `cast` and `default` parameters.
@@ -66,18 +78,21 @@ class Env(object):
     ENVIRON = os.environ
     NOTSET = NoValue()
     BOOLEAN_TRUE_STRINGS = ('true', 'on', 'ok', 'y', 'yes', '1')
-    URL_CLASS = urllib.parse.ParseResult
+    URL_CLASS = ParseResult
     DEFAULT_DATABASE_ENV = 'DATABASE_URL'
+
+    POSTGRES_FAMILY = ['postgres', 'postgresql', 'psql', 'pgsql', 'postgis']
     DB_SCHEMES = {
-        'postgres': 'django.db.backends.postgresql_psycopg2',
-        'postgresql': 'django.db.backends.postgresql_psycopg2',
-        'psql': 'django.db.backends.postgresql_psycopg2',
-        'pgsql': 'django.db.backends.postgresql_psycopg2',
+        'postgres': DJANGO_POSTGRES,
+        'postgresql': DJANGO_POSTGRES,
+        'psql': DJANGO_POSTGRES,
+        'pgsql': DJANGO_POSTGRES,
         'postgis': 'django.contrib.gis.db.backends.postgis',
         'mysql': 'django.db.backends.mysql',
         'mysql2': 'django.db.backends.mysql',
         'mysql-connector': 'mysql.connector.django',
         'mysqlgis': 'django.contrib.gis.db.backends.mysql',
+        'mssql': 'sql_server.pyodbc',
         'oracle': 'django.db.backends.oracle',
         'pyodbc': 'sql_server.pyodbc',
         'redshift': 'django_redshift_backend',
@@ -85,7 +100,12 @@ class Env(object):
         'sqlite': 'django.db.backends.sqlite3',
         'ldap': 'ldapdb.backends.ldap',
     }
-    _DB_BASE_OPTIONS = ['CONN_MAX_AGE', 'ATOMIC_REQUESTS', 'AUTOCOMMIT']
+    _DB_BASE_OPTIONS = [
+        'CONN_MAX_AGE',
+        'ATOMIC_REQUESTS',
+        'AUTOCOMMIT',
+        'DISABLE_SERVER_SIDE_CURSORS',
+    ]
 
     DEFAULT_CACHE_ENV = 'CACHE_URL'
     CACHE_SCHEMES = {
@@ -97,8 +117,15 @@ class Env(object):
         'pymemcache': 'django.core.cache.backends.memcached.PyLibMCCache',
         'rediscache': REDIS_DRIVER,
         'redis': REDIS_DRIVER,
+        'rediss': REDIS_DRIVER,
     }
-    _CACHE_BASE_OPTIONS = ['TIMEOUT', 'KEY_PREFIX', 'VERSION', 'KEY_FUNCTION', 'BINARY']
+    _CACHE_BASE_OPTIONS = [
+        'TIMEOUT',
+        'KEY_PREFIX',
+        'VERSION',
+        'KEY_FUNCTION',
+        'BINARY',
+    ]
 
     DEFAULT_EMAIL_ENV = 'EMAIL_URL'
     EMAIL_SCHEMES = {
@@ -115,8 +142,10 @@ class Env(object):
 
     DEFAULT_SEARCH_ENV = 'SEARCH_URL'
     SEARCH_SCHEMES = {
-        "elasticsearch": "haystack.backends.elasticsearch_backend.ElasticsearchSearchEngine",
-        "elasticsearch2": "haystack.backends.elasticsearch2_backend.Elasticsearch2SearchEngine",
+        "elasticsearch": "haystack.backends.elasticsearch_backend."
+                         "ElasticsearchSearchEngine",
+        "elasticsearch2": "haystack.backends.elasticsearch2_backend."
+                          "Elasticsearch2SearchEngine",
         "solr": "haystack.backends.solr_backend.SolrEngine",
         "whoosh": "haystack.backends.whoosh_backend.WhooshEngine",
         "xapian": "haystack.backends.xapian_backend.XapianEngine",
@@ -124,21 +153,30 @@ class Env(object):
     }
 
     def __init__(self, **scheme):
+        self.smart_cast = True
         self.scheme = scheme
 
     def __call__(self, var, cast=None, default=NOTSET, parse_default=False):
-        return self.get_value(var, cast=cast, default=default, parse_default=parse_default)
+        return self.get_value(
+            var,
+            cast=cast,
+            default=default,
+            parse_default=parse_default
+        )
 
     def __contains__(self, var):
         return var in self.ENVIRON
 
     # Shortcuts
 
-    def str(self, var, default=NOTSET):
+    def str(self, var, default=NOTSET, multiline=False):
         """
         :rtype: str
         """
-        return self.get_value(var, default=default)
+        value = self.get_value(var, cast=str, default=default)
+        if multiline:
+            return value.replace('\\n', '\n')
+        return value
 
     def unicode(self, var, default=NOTSET):
         """Helper for python2
@@ -146,6 +184,12 @@ class Env(object):
         """
         return self.get_value(var, cast=str, default=default)
 
+    def bytes(self, var, default=NOTSET, encoding='utf8'):
+        """
+        :rtype: bytes
+        """
+        return self.get_value(var, cast=str).encode(encoding)
+
     def bool(self, var, default=NOTSET):
         """
         :rtype: bool
@@ -174,13 +218,21 @@ class Env(object):
         """
         :rtype: list
         """
-        return self.get_value(var, cast=list if not cast else [cast], default=default)
+        return self.get_value(
+            var,
+            cast=list if not cast else [cast],
+            default=default
+        )
 
     def tuple(self, var, cast=None, default=NOTSET):
         """
         :rtype: tuple
         """
-        return self.get_value(var, cast=tuple if not cast else (cast,), default=default)
+        return self.get_value(
+            var,
+            cast=tuple if not cast else (cast,),
+            default=default
+        )
 
     def dict(self, var, cast=dict, default=NOTSET):
         """
@@ -192,30 +244,50 @@ class Env(object):
         """
         :rtype: urlparse.ParseResult
         """
-        return self.get_value(var, cast=urllib.parse.urlparse, default=default, parse_default=True)
+        return self.get_value(
+            var,
+            cast=urlparse,
+            default=default,
+            parse_default=True
+        )
 
     def db_url(self, var=DEFAULT_DATABASE_ENV, default=NOTSET, engine=None):
         """Returns a config dictionary, defaulting to DATABASE_URL.
 
+        The db method is an alias for db_url.
+
         :rtype: dict
         """
-        return self.db_url_config(self.get_value(var, default=default), engine=engine)
+        return self.db_url_config(
+            self.get_value(var, default=default),
+            engine=engine
+        )
     db = db_url
 
     def cache_url(self, var=DEFAULT_CACHE_ENV, default=NOTSET, backend=None):
         """Returns a config dictionary, defaulting to CACHE_URL.
 
+        The cache method is an alias for cache_url.
+
         :rtype: dict
         """
-        return self.cache_url_config(self.url(var, default=default), backend=backend)
+        return self.cache_url_config(
+            self.url(var, default=default),
+            backend=backend
+        )
     cache = cache_url
 
     def email_url(self, var=DEFAULT_EMAIL_ENV, default=NOTSET, backend=None):
         """Returns a config dictionary, defaulting to EMAIL_URL.
 
+        The email method is an alias for email_url.
+
         :rtype: dict
         """
-        return self.email_url_config(self.url(var, default=default), backend=backend)
+        return self.email_url_config(
+            self.url(var, default=default),
+            backend=backend
+        )
     email = email_url
 
     def search_url(self, var=DEFAULT_SEARCH_ENV, default=NOTSET, engine=None):
@@ -223,7 +295,10 @@ class Env(object):
 
         :rtype: dict
         """
-        return self.search_url_config(self.url(var, default=default), engine=engine)
+        return self.search_url_config(
+            self.url(var, default=default),
+            engine=engine
+        )
 
     def path(self, var, default=NOTSET, **kwargs):
         """
@@ -242,7 +317,7 @@ class Env(object):
         :returns: Value from environment or default (if set)
         """
 
-        logger.debug("get '{0}' casted as '{1}' with default '{2}'".format(
+        logger.debug("get '{}' casted as '{}' with default '{}'".format(
             var, cast, default
         ))
 
@@ -271,7 +346,7 @@ class Env(object):
             value = self.ENVIRON[var]
         except KeyError:
             if default is self.NOTSET:
-                error_msg = "Set the {0} environment variable".format(var)
+                error_msg = "Set the {} environment variable".format(var)
                 raise ImproperlyConfigured(error_msg)
 
             value = default
@@ -281,6 +356,12 @@ class Env(object):
             value = value.lstrip('$')
             value = self.get_value(value, cast=cast, default=default)
 
+        # Smart casting
+        if self.smart_cast:
+            if cast is None and default is not None and \
+                    not isinstance(default, NoValue):
+                cast = type(default)
+
         if value != default or (parse_default and value):
             value = self.parse_value(value, cast)
 
@@ -316,7 +397,10 @@ class Env(object):
             value = dict(map(
                 lambda kv: (
                     key_cast(kv[0]),
-                    cls.parse_value(kv[1], value_cast_by_key.get(kv[0], value_cast))
+                    cls.parse_value(
+                        kv[1],
+                        value_cast_by_key.get(kv[0], value_cast)
+                    )
                 ),
                 [val.split('=') for val in value.split(';') if val]
             ))
@@ -330,12 +414,13 @@ class Env(object):
         elif cast is float:
             # clean string
             float_str = re.sub(r'[^\d,\.]', '', value)
-            # split for avoid thousand separator and different locale comma/dot symbol
+            # split for avoid thousand separator and different
+            # locale comma/dot symbol
             parts = re.split(r'[,\.]', float_str)
             if len(parts) == 1:
                 float_str = parts[0]
             else:
-                float_str = "{0}.{1}".format(''.join(parts[0:-1]), parts[-1])
+                float_str = "{}.{}".format(''.join(parts[0:-1]), parts[-1])
             value = float(float_str)
         else:
             value = cast(value)
@@ -344,18 +429,14 @@ class Env(object):
     @classmethod
     def db_url_config(cls, url, engine=None):
         """Pulled from DJ-Database-URL, parse an arbitrary Database URL.
-        Support currently exists for PostgreSQL, PostGIS, MySQL, Oracle and SQLite.
 
-        SQLite connects to file based databases. The same URL format is used, omitting the hostname,
-        and using the "file" portion as the filename of the database.
-        This has the effect of four slashes being present for an absolute file path:
-
-        >>> from environ import Env
-        >>> Env.db_url_config('sqlite:////full/path/to/your/file.sqlite')
-        {'ENGINE': 'django.db.backends.sqlite3', 'HOST': '', 'NAME': '/full/path/to/your/file.sqlite', 'PASSWORD': '', 'PORT': '', 'USER': ''}
-        >>> Env.db_url_config('postgres://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn')
-        {'ENGINE': 'django.db.backends.postgresql_psycopg2', 'HOST': 'ec2-107-21-253-135.compute-1.amazonaws.com', 'NAME': 'd8r82722r2kuvn', 'PASSWORD': 'wegauwhgeuioweg', 'PORT': 5431, 'USER': 'uf07k1i6d8ia0v'}
+        Support currently exists for PostgreSQL, PostGIS, MySQL, Oracle and
+        SQLite.
 
+        SQLite connects to file based databases. The same URL format is used,
+        omitting the hostname, and using the "file" portion as the filename of
+        the database. This has the effect of four slashes being present for an
+        absolute file path.
         """
         if not isinstance(url, cls.URL_CLASS):
             if url == 'sqlite://:memory:':
@@ -367,20 +448,28 @@ class Env(object):
                     'NAME': ':memory:'
                 }
                 # note: no other settings are required for sqlite
-            url = urllib.parse.urlparse(url)
+            url = urlparse(url)
 
         config = {}
 
         # Remove query strings.
         path = url.path[1:]
-        path = urllib.parse.unquote_plus(path.split('?', 2)[0])
+        path = unquote_plus(path.split('?', 2)[0])
 
-        # if we are using sqlite and we have no path, then assume we
-        # want an in-memory database (this is the behaviour of sqlalchemy)
-        if url.scheme == 'sqlite' and path == '':
-            path = ':memory:'
+        if url.scheme == 'sqlite':
+            if path == '':
+                # if we are using sqlite and we have no path, then assume we
+                # want an in-memory database (this is the behaviour of
+                # sqlalchemy)
+                path = ':memory:'
+            if url.netloc:
+                warnings.warn('SQLite URL contains host component %r, '
+                              'it will be ignored' % url.netloc, stacklevel=3)
         if url.scheme == 'ldap':
-            path = '{scheme}://{hostname}'.format(scheme=url.scheme, hostname=url.hostname)
+            path = '{scheme}://{hostname}'.format(
+                scheme=url.scheme,
+                hostname=url.hostname,
+            )
             if url.port:
                 path += ':{port}'.format(port=url.port)
 
@@ -393,6 +482,9 @@ class Env(object):
             'PORT': _cast_int(url.port) or '',
         })
 
+        if url.scheme in cls.POSTGRES_FAMILY and path.startswith('/'):
+            config['HOST'], config['NAME'] = path.rsplit('/', 1)
+
         if url.scheme == 'oracle' and path == '':
             config['NAME'] = config['HOST']
             config['HOST'] = ''
@@ -406,20 +498,23 @@ class Env(object):
 
         if url.query:
             config_options = {}
-            for k, v in urllib.parse.parse_qs(url.query).items():
+            for k, v in parse_qs(url.query).items():
                 if k.upper() in cls._DB_BASE_OPTIONS:
-                    config.update({k.upper(): _cast_int(v[0])})
+                    config.update({k.upper(): _cast(v[0])})
                 else:
                     config_options.update({k: _cast_int(v[0])})
             config['OPTIONS'] = config_options
 
         if engine:
             config['ENGINE'] = engine
-        if url.scheme in Env.DB_SCHEMES:
-            config['ENGINE'] = Env.DB_SCHEMES[url.scheme]
+        else:
+            config['ENGINE'] = url.scheme
+
+        if config['ENGINE'] in Env.DB_SCHEMES:
+            config['ENGINE'] = Env.DB_SCHEMES[config['ENGINE']]
 
         if not config.get('ENGINE', False):
-            warnings.warn("Engine not recognized from url: {0}".format(config))
+            warnings.warn("Engine not recognized from url: {}".format(config))
             return {}
 
         return config
@@ -432,7 +527,16 @@ class Env(object):
         :param backend:
         :return:
         """
-        url = urllib.parse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
+        if not isinstance(url, cls.URL_CLASS):
+            if not url:
+                return {}
+            else:
+                url = urlparse(url)
+
+        if url.scheme not in cls.CACHE_SCHEMES:
+            raise ImproperlyConfigured(
+                'Invalid cache schema {}'.format(url.scheme)
+            )
 
         location = url.netloc.split(',')
         if len(location) == 1:
@@ -458,13 +562,17 @@ class Env(object):
                 scheme = url.scheme.replace('cache', '')
             else:
                 scheme = 'unix'
-            locations = [scheme + '://' + loc + url.path for loc in url.netloc.split(',')]
-            config['LOCATION'] = locations[0] if len(locations) == 1 else locations
+            locations = [scheme + '://' + loc + url.path
+                         for loc in url.netloc.split(',')]
+            if len(locations) == 1:
+                config['LOCATION'] = locations[0]
+            else:
+                config['LOCATION'] = locations
 
         if url.query:
             config_options = {}
-            for k, v in urllib.parse.parse_qs(url.query).items():
-                opt = {k.upper(): _cast_int(v[0])}
+            for k, v in parse_qs(url.query).items():
+                opt = {k.upper(): _cast(v[0])}
                 if k.upper() in cls._CACHE_BASE_OPTIONS:
                     config.update(opt)
                 else:
@@ -482,11 +590,11 @@ class Env(object):
 
         config = {}
 
-        url = urllib.parse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
+        url = urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
 
         # Remove query strings
         path = url.path[1:]
-        path = urllib.parse.unquote_plus(path.split('?', 2)[0])
+        path = unquote_plus(path.split('?', 2)[0])
 
         # Update with environment configuration
         config.update({
@@ -511,7 +619,7 @@ class Env(object):
 
         if url.query:
             config_options = {}
-            for k, v in urllib.parse.parse_qs(url.query).items():
+            for k, v in parse_qs(url.query).items():
                 opt = {k.upper(): _cast_int(v[0])}
                 if k.upper() in cls._EMAIL_BASE_OPTIONS:
                     config.update(opt)
@@ -525,26 +633,35 @@ class Env(object):
     def search_url_config(cls, url, engine=None):
         config = {}
 
-        url = urllib.parse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
+        url = urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
 
         # Remove query strings.
         path = url.path[1:]
-        path = urllib.parse.unquote_plus(path.split('?', 2)[0])
+        path = unquote_plus(path.split('?', 2)[0])
 
         if url.scheme not in cls.SEARCH_SCHEMES:
-            raise ImproperlyConfigured('Invalid search schema %s' % url.scheme)
+            raise ImproperlyConfigured(
+                'Invalid search schema %s' % url.scheme
+            )
         config["ENGINE"] = cls.SEARCH_SCHEMES[url.scheme]
 
         # check commons params
         params = {}
         if url.query:
-            params = urllib.parse.parse_qs(url.query)
+            params = parse_qs(url.query)
             if 'EXCLUDED_INDEXES' in params.keys():
-                config['EXCLUDED_INDEXES'] = params['EXCLUDED_INDEXES'][0].split(',')
+                config['EXCLUDED_INDEXES'] \
+                    = params['EXCLUDED_INDEXES'][0].split(',')
             if 'INCLUDE_SPELLING' in params.keys():
-                config['INCLUDE_SPELLING'] = cls.parse_value(params['INCLUDE_SPELLING'][0], bool)
+                config['INCLUDE_SPELLING'] = cls.parse_value(
+                    params['INCLUDE_SPELLING'][0],
+                    bool
+                )
             if 'BATCH_SIZE' in params.keys():
-                config['BATCH_SIZE'] = cls.parse_value(params['BATCH_SIZE'][0], int)
+                config['BATCH_SIZE'] = cls.parse_value(
+                    params['BATCH_SIZE'][0],
+                    int
+                )
 
         if url.scheme == 'simple':
             return config
@@ -557,7 +674,9 @@ class Env(object):
             path = path[:-1]
 
         if url.scheme == 'solr':
-            config['URL'] = urllib.parse.urlunparse(('http',) + url[1:2] + (path,) + ('', '', ''))
+            config['URL'] = urlunparse(
+                ('http',) + url[1:2] + (path,) + ('', '', '')
+            )
             if 'TIMEOUT' in params.keys():
                 config['TIMEOUT'] = cls.parse_value(params['TIMEOUT'][0], int)
             return config
@@ -573,7 +692,9 @@ class Env(object):
                 path = ""
                 index = split[0]
 
-            config['URL'] = urllib.parse.urlunparse(('http',) + url[1:2] + (path,) + ('', '', ''))
+            config['URL'] = urlunparse(
+                ('http',) + url[1:2] + (path,) + ('', '', '')
+            )
             if 'TIMEOUT' in params.keys():
                 config['TIMEOUT'] = cls.parse_value(params['TIMEOUT'][0], int)
             config['INDEX_NAME'] = index
@@ -585,7 +706,10 @@ class Env(object):
             if 'STORAGE' in params.keys():
                 config['STORAGE'] = params['STORAGE'][0]
             if 'POST_LIMIT' in params.keys():
-                config['POST_LIMIT'] = cls.parse_value(params['POST_LIMIT'][0], int)
+                config['POST_LIMIT'] = cls.parse_value(
+                    params['POST_LIMIT'][0],
+                    int
+                )
         elif url.scheme == 'xapian':
             if 'FLAGS' in params.keys():
                 config['FLAGS'] = params['FLAGS'][0]
@@ -599,16 +723,20 @@ class Env(object):
     def read_env(cls, env_file=None, **overrides):
         """Read a .env file into os.environ.
 
-        If not given a path to a dotenv path, does filthy magic stack backtracking
-        to find manage.py and then find the dotenv.
+        If not given a path to a dotenv path, does filthy magic stack
+        backtracking to find the dotenv in the same directory as the file that
+        called read_env.
 
-        http://www.wellfireinteractive.com/blog/easier-12-factor-django/
-
-        https://gist.github.com/bennylope/2999704
+        Refs:
+        - http://www.wellfireinteractive.com/blog/easier-12-factor-django/
+        - https://gist.github.com/bennylope/2999704
         """
         if env_file is None:
             frame = sys._getframe()
-            env_file = os.path.join(os.path.dirname(frame.f_back.f_code.co_filename), '.env')
+            env_file = os.path.join(
+                os.path.dirname(frame.f_back.f_code.co_filename),
+                '.env'
+            )
             if not os.path.exists(env_file):
                 warnings.warn(
                     "%s doesn't exist - if you're not configuring your "
@@ -616,18 +744,22 @@ class Env(object):
                 return
 
         try:
-            with open(env_file) if isinstance(env_file, string_types) else env_file as f:
-                content = f.read()
-        except IOError:
+            if isinstance(env_file, Openable):
+                with open(env_file) as f:
+                    content = f.read()
+            else:
+                with env_file as f:
+                    content = f.read()
+        except OSError:
             warnings.warn(
                 "Error reading %s - if you're not configuring your "
                 "environment separately, check this." % env_file)
             return
 
-        logger.debug('Read environment variables from: {0}'.format(env_file))
+        logger.debug('Read environment variables from: {}'.format(env_file))
 
         for line in content.splitlines():
-            m1 = re.match(r'\A([A-Za-z_0-9]+)=(.*)\Z', line)
+            m1 = re.match(r'\A(?:export )?([A-Za-z_0-9]+)=(.*)\Z', line)
             if m1:
                 key, val = m1.group(1), m1.group(2)
                 m2 = re.match(r"\A'(.*)'\Z", val)
@@ -643,31 +775,9 @@ class Env(object):
             cls.ENVIRON.setdefault(key, value)
 
 
-class Path(object):
+class Path:
 
-    """Inspired to Django Two-scoops, handling File Paths in Settings.
-
-        >>> from environ import Path
-        >>> root = Path('/home')
-        >>> root, root(), root('dev')
-        (<Path:/home>, '/home', '/home/dev')
-        >>> root == Path('/home')
-        True
-        >>> root in Path('/'), root not in Path('/other/path')
-        (True, True)
-        >>> root('dev', 'not_existing_dir', required=True)
-        Traceback (most recent call last):
-        environ.environ.ImproperlyConfigured: Create required path: /home/not_existing_dir
-        >>> public = root.path('public')
-        >>> public, public.root, public('styles')
-        (<Path:/home/public>, '/home/public', '/home/public/styles')
-        >>> assets, scripts = public.path('assets'), public.path('assets', 'scripts')
-        >>> assets.root, scripts.root
-        ('/home/public/assets', '/home/public/assets/scripts')
-        >>> assets + 'styles', str(assets + 'styles'), ~assets
-        (<Path:/home/public/assets/styles>, '/home/public/assets/styles', <Path:/home/public>)
-
-    """
+    """Inspired to Django Two-scoops, handling File Paths in Settings."""
 
     def path(self, *paths, **kwargs):
         """Create new Path based on self.root and provided paths.
@@ -696,7 +806,7 @@ class Path(object):
 
     def __init__(self, start='', *paths, **kwargs):
 
-        super(Path, self).__init__()
+        super().__init__()
 
         if kwargs.get('is_file', False):
             start = os.path.dirname(start)
@@ -718,12 +828,14 @@ class Path(object):
         return not self.__eq__(other)
 
     def __add__(self, other):
-        return Path(self.__root__, other if not isinstance(other, Path) else other.__root__)
+        if not isinstance(other, Path):
+            return Path(self.__root__, other)
+        return Path(self.__root__, other.__root__)
 
     def __sub__(self, other):
         if isinstance(other, int):
             return self.path('../' * other)
-        elif isinstance(other, string_types):
+        elif isinstance(other, str):
             if self.__root__.endswith(other):
                 return Path(self.__root__.rstrip(other))
         raise TypeError(
@@ -743,7 +855,7 @@ class Path(object):
         return item.__root__.startswith(base_path)
 
     def __repr__(self):
-        return "<Path:{0}>".format(self.__root__)
+        return "<Path:{}>".format(self.__root__)
 
     def __str__(self):
         return self.__root__
@@ -754,6 +866,9 @@ class Path(object):
     def __getitem__(self, *args, **kwargs):
         return self.__str__().__getitem__(*args, **kwargs)
 
+    def __fspath__(self):
+        return self.__str__()
+
     def rfind(self, *args, **kwargs):
         return self.__str__().rfind(*args, **kwargs)
 
@@ -765,14 +880,14 @@ class Path(object):
         absolute_path = os.path.abspath(os.path.join(base, *paths))
         if kwargs.get('required', False) and not os.path.exists(absolute_path):
             raise ImproperlyConfigured(
-                "Create required path: {0}".format(absolute_path))
+                "Create required path: {}".format(absolute_path))
         return absolute_path
 
 
 def register_scheme(scheme):
-    for method in dir(urllib.parse):
+    for method in dir(urlparselib):
         if method.startswith('uses_'):
-            getattr(urllib.parse, method).append(scheme)
+            getattr(urlparselib, method).append(scheme)
 
 
 def register_schemes(schemes):
